/* src/include/ivstream.h.  Generated from ivstream.h.in by configure.  */
#ifndef ivstream_h
#define ivstream_h

/* The standard is <iostream> */
#define HAVE_SSTREAM /**/

/* Define if the openmodes output,append,input are not defined */
/* #undef NO_OUTPUT_OPENMODE */

#include <ivstrm.h>

#endif
